!! ##############################################################################################
!!
!! Copyright 2012-2020 CNRS, INPT
!! Copyright 2013-2015 UPS
!!  
!! This file is part of qr_mumps.
!!  
!! qr_mumps is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!  
!! qr_mumps is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!  
!! You can find a copy of the GNU Lesser General Public License
!! in the qr_mumps/doc directory.
!!
!! ##############################################################################################


!! ##############################################################################################
!!
!! This file contains a module with all the generic interfaces for the typed methods routines.
!!
!! ##############################################################################################


!> This module contains generic methods
module cqrm_methods_mod

  interface qrm_spbackslash
     procedure               ::  cqrm_spbackslash2d, cqrm_spbackslash1d
  end interface qrm_spbackslash

  interface cqrm_spbackslash
     subroutine cqrm_spbackslash2d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_spbackslash2d
     subroutine cqrm_spbackslash1d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_spbackslash1d
  end interface cqrm_spbackslash

  
  interface qrm_spposv
     procedure               ::  cqrm_spposv2d, cqrm_spposv1d
  end interface qrm_spposv

  interface cqrm_spposv
     subroutine cqrm_spposv2d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_spposv2d
     subroutine cqrm_spposv1d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_spposv1d
  end interface cqrm_spposv

  interface qrm_least_squares
     procedure               ::  cqrm_least_squares2d, cqrm_least_squares1d
  end interface qrm_least_squares

  interface cqrm_least_squares
     subroutine cqrm_least_squares2d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_least_squares2d
     subroutine cqrm_least_squares1d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_least_squares1d
  end interface cqrm_least_squares
  


  interface qrm_min_norm
     procedure               :: cqrm_min_norm2d, cqrm_min_norm1d
  end interface qrm_min_norm

  interface cqrm_min_norm
     subroutine cqrm_min_norm2d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_min_norm2d
     subroutine cqrm_min_norm1d(qrm_mat, b, x, cperm, info)
       use cqrm_spmat_mod
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine cqrm_min_norm1d
  end interface cqrm_min_norm
  

  interface qrm_residual_norm
     procedure               :: cqrm_residual_norm2d, cqrm_residual_norm1d
  end interface qrm_residual_norm

  interface cqrm_residual_norm
     subroutine cqrm_residual_norm2d(qrm_mat, b, x, nrm, info)
       use cqrm_spmat_mod
       real(r32)             :: nrm(:)
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine cqrm_residual_norm2d
     subroutine cqrm_residual_norm1d(qrm_mat, b, x, nrm, info)
       use cqrm_spmat_mod
       real(r32)             :: nrm
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine cqrm_residual_norm1d
  end interface cqrm_residual_norm



  interface qrm_residual_orth
     procedure               :: cqrm_residual_orth2d, cqrm_residual_orth1d
     procedure               :: cqrm_residual_and_orth2d, cqrm_residual_and_orth1d
  end interface qrm_residual_orth

  interface cqrm_residual_orth
     subroutine cqrm_residual_orth2d(qrm_mat, r, nrm, info)
       use cqrm_spmat_mod
       real(r32)             :: nrm(:)
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: r(:,:)
       integer, optional     :: info
     end subroutine cqrm_residual_orth2d
     subroutine cqrm_residual_orth1d(qrm_mat, r, nrm, info)
       use cqrm_spmat_mod
       real(r32)             :: nrm
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: r(:)
       integer, optional     :: info
     end subroutine cqrm_residual_orth1d
     subroutine cqrm_residual_and_orth2d(qrm_mat, b, x, nrm, info)
       use cqrm_spmat_mod
       real(r32)             :: nrm(:)
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine cqrm_residual_and_orth2d
     subroutine cqrm_residual_and_orth1d(qrm_mat, b, x, nrm, info)
       use cqrm_spmat_mod
       real(r32)             :: nrm
       type(cqrm_spmat_type) :: qrm_mat
       complex(r32)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine cqrm_residual_and_orth1d
  end interface cqrm_residual_orth


  interface qrm_get_r
     subroutine cqrm_get_r(qrm_spfct, r, info)
       use cqrm_spmat_mod
       use cqrm_spfct_mod
       type(cqrm_spfct_type)          :: qrm_spfct
       type(cqrm_spmat_type)          :: r
       integer, optional              :: info
     end subroutine cqrm_get_r
  end interface qrm_get_r
  
end module cqrm_methods_mod
