!! ##############################################################################################
!!
!! Copyright 2012-2020 CNRS, INPT
!! Copyright 2013-2015 UPS
!!  
!! This file is part of qr_mumps.
!!  
!! qr_mumps is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!  
!! qr_mumps is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!  
!! You can find a copy of the GNU Lesser General Public License
!! in the qr_mumps/doc directory.
!!
!! ##############################################################################################


!! ##############################################################################################
!!
!! This file contains a module with all the generic interfaces for the typed methods routines.
!!
!! ##############################################################################################


!> This module contains generic methods
module dqrm_methods_mod

  interface qrm_spbackslash
     procedure               ::  dqrm_spbackslash2d, dqrm_spbackslash1d
  end interface qrm_spbackslash

  interface dqrm_spbackslash
     subroutine dqrm_spbackslash2d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_spbackslash2d
     subroutine dqrm_spbackslash1d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_spbackslash1d
  end interface dqrm_spbackslash

  
  interface qrm_spposv
     procedure               ::  dqrm_spposv2d, dqrm_spposv1d
  end interface qrm_spposv

  interface dqrm_spposv
     subroutine dqrm_spposv2d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_spposv2d
     subroutine dqrm_spposv1d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_spposv1d
  end interface dqrm_spposv

  interface qrm_least_squares
     procedure               ::  dqrm_least_squares2d, dqrm_least_squares1d
  end interface qrm_least_squares

  interface dqrm_least_squares
     subroutine dqrm_least_squares2d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_least_squares2d
     subroutine dqrm_least_squares1d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_least_squares1d
  end interface dqrm_least_squares
  


  interface qrm_min_norm
     procedure               :: dqrm_min_norm2d, dqrm_min_norm1d
  end interface qrm_min_norm

  interface dqrm_min_norm
     subroutine dqrm_min_norm2d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_min_norm2d
     subroutine dqrm_min_norm1d(qrm_mat, b, x, cperm, info)
       use dqrm_spmat_mod
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine dqrm_min_norm1d
  end interface dqrm_min_norm
  

  interface qrm_residual_norm
     procedure               :: dqrm_residual_norm2d, dqrm_residual_norm1d
  end interface qrm_residual_norm

  interface dqrm_residual_norm
     subroutine dqrm_residual_norm2d(qrm_mat, b, x, nrm, info)
       use dqrm_spmat_mod
       real(r64)             :: nrm(:)
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine dqrm_residual_norm2d
     subroutine dqrm_residual_norm1d(qrm_mat, b, x, nrm, info)
       use dqrm_spmat_mod
       real(r64)             :: nrm
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine dqrm_residual_norm1d
  end interface dqrm_residual_norm



  interface qrm_residual_orth
     procedure               :: dqrm_residual_orth2d, dqrm_residual_orth1d
     procedure               :: dqrm_residual_and_orth2d, dqrm_residual_and_orth1d
  end interface qrm_residual_orth

  interface dqrm_residual_orth
     subroutine dqrm_residual_orth2d(qrm_mat, r, nrm, info)
       use dqrm_spmat_mod
       real(r64)             :: nrm(:)
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: r(:,:)
       integer, optional     :: info
     end subroutine dqrm_residual_orth2d
     subroutine dqrm_residual_orth1d(qrm_mat, r, nrm, info)
       use dqrm_spmat_mod
       real(r64)             :: nrm
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: r(:)
       integer, optional     :: info
     end subroutine dqrm_residual_orth1d
     subroutine dqrm_residual_and_orth2d(qrm_mat, b, x, nrm, info)
       use dqrm_spmat_mod
       real(r64)             :: nrm(:)
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine dqrm_residual_and_orth2d
     subroutine dqrm_residual_and_orth1d(qrm_mat, b, x, nrm, info)
       use dqrm_spmat_mod
       real(r64)             :: nrm
       type(dqrm_spmat_type) :: qrm_mat
       real(r64)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine dqrm_residual_and_orth1d
  end interface dqrm_residual_orth


  interface qrm_get_r
     subroutine dqrm_get_r(qrm_spfct, r, info)
       use dqrm_spmat_mod
       use dqrm_spfct_mod
       type(dqrm_spfct_type)          :: qrm_spfct
       type(dqrm_spmat_type)          :: r
       integer, optional              :: info
     end subroutine dqrm_get_r
  end interface qrm_get_r
  
end module dqrm_methods_mod
