!! ##############################################################################################
!!
!! Copyright 2012-2020 CNRS, INPT
!! Copyright 2013-2015 UPS
!!  
!! This file is part of qr_mumps.
!!  
!! qr_mumps is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!  
!! qr_mumps is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!  
!! You can find a copy of the GNU Lesser General Public License
!! in the qr_mumps/doc directory.
!!
!! ##############################################################################################


!! ##############################################################################################
!!
!! This file contains a module with all the generic interfaces for the typed methods routines.
!!
!! ##############################################################################################


!> This module contains generic methods
module sqrm_methods_mod

  interface qrm_spbackslash
     procedure               ::  sqrm_spbackslash2d, sqrm_spbackslash1d
  end interface qrm_spbackslash

  interface sqrm_spbackslash
     subroutine sqrm_spbackslash2d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_spbackslash2d
     subroutine sqrm_spbackslash1d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_spbackslash1d
  end interface sqrm_spbackslash

  
  interface qrm_spposv
     procedure               ::  sqrm_spposv2d, sqrm_spposv1d
  end interface qrm_spposv

  interface sqrm_spposv
     subroutine sqrm_spposv2d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_spposv2d
     subroutine sqrm_spposv1d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_spposv1d
  end interface sqrm_spposv

  interface qrm_least_squares
     procedure               ::  sqrm_least_squares2d, sqrm_least_squares1d
  end interface qrm_least_squares

  interface sqrm_least_squares
     subroutine sqrm_least_squares2d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_least_squares2d
     subroutine sqrm_least_squares1d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_least_squares1d
  end interface sqrm_least_squares
  


  interface qrm_min_norm
     procedure               :: sqrm_min_norm2d, sqrm_min_norm1d
  end interface qrm_min_norm

  interface sqrm_min_norm
     subroutine sqrm_min_norm2d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_min_norm2d
     subroutine sqrm_min_norm1d(qrm_mat, b, x, cperm, info)
       use sqrm_spmat_mod
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine sqrm_min_norm1d
  end interface sqrm_min_norm
  

  interface qrm_residual_norm
     procedure               :: sqrm_residual_norm2d, sqrm_residual_norm1d
  end interface qrm_residual_norm

  interface sqrm_residual_norm
     subroutine sqrm_residual_norm2d(qrm_mat, b, x, nrm, info)
       use sqrm_spmat_mod
       real(r32)             :: nrm(:)
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine sqrm_residual_norm2d
     subroutine sqrm_residual_norm1d(qrm_mat, b, x, nrm, info)
       use sqrm_spmat_mod
       real(r32)             :: nrm
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine sqrm_residual_norm1d
  end interface sqrm_residual_norm



  interface qrm_residual_orth
     procedure               :: sqrm_residual_orth2d, sqrm_residual_orth1d
     procedure               :: sqrm_residual_and_orth2d, sqrm_residual_and_orth1d
  end interface qrm_residual_orth

  interface sqrm_residual_orth
     subroutine sqrm_residual_orth2d(qrm_mat, r, nrm, info)
       use sqrm_spmat_mod
       real(r32)             :: nrm(:)
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: r(:,:)
       integer, optional     :: info
     end subroutine sqrm_residual_orth2d
     subroutine sqrm_residual_orth1d(qrm_mat, r, nrm, info)
       use sqrm_spmat_mod
       real(r32)             :: nrm
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: r(:)
       integer, optional     :: info
     end subroutine sqrm_residual_orth1d
     subroutine sqrm_residual_and_orth2d(qrm_mat, b, x, nrm, info)
       use sqrm_spmat_mod
       real(r32)             :: nrm(:)
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine sqrm_residual_and_orth2d
     subroutine sqrm_residual_and_orth1d(qrm_mat, b, x, nrm, info)
       use sqrm_spmat_mod
       real(r32)             :: nrm
       type(sqrm_spmat_type) :: qrm_mat
       real(r32)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine sqrm_residual_and_orth1d
  end interface sqrm_residual_orth


  interface qrm_get_r
     subroutine sqrm_get_r(qrm_spfct, r, info)
       use sqrm_spmat_mod
       use sqrm_spfct_mod
       type(sqrm_spfct_type)          :: qrm_spfct
       type(sqrm_spmat_type)          :: r
       integer, optional              :: info
     end subroutine sqrm_get_r
  end interface qrm_get_r
  
end module sqrm_methods_mod
