!! ##############################################################################################
!!
!! Copyright 2012-2020 CNRS, INPT
!! Copyright 2013-2015 UPS
!!  
!! This file is part of qr_mumps.
!!  
!! qr_mumps is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!  
!! qr_mumps is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!  
!! You can find a copy of the GNU Lesser General Public License
!! in the qr_mumps/doc directory.
!!
!! ##############################################################################################


!! ##############################################################################################
!!
!! This file contains a module with generic interfaces for all the solve typed routines
!!
!! ##############################################################################################


!> @brief This module contains all the interfaces for the typed routines
!! in the solve phase
module sqrm_solve_mod
  
  interface qrm_apply_async
     subroutine sqrm_apply_async(qrm_dscr, qrm_spfct, transp, qrm_sdata_b)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       use qrm_dscr_mod
       type(qrm_dscr_type)                  :: qrm_dscr
       type(sqrm_spfct_type), intent(in)    :: qrm_spfct
       type(sqrm_sdata_type)                :: qrm_sdata_b
       character(len=*), intent(in)         :: transp
     end subroutine sqrm_apply_async
  end interface qrm_apply_async

  interface qrm_solve_async
     subroutine sqrm_solve_async(qrm_dscr, qrm_spfct, transp, qrm_sdata_b, qrm_sdata_x)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       use qrm_dscr_mod
       type(qrm_dscr_type)                  :: qrm_dscr
       type(sqrm_spfct_type), intent(in)    :: qrm_spfct
       type(sqrm_sdata_type)                :: qrm_sdata_b, qrm_sdata_x
       character(len=*), intent(in)         :: transp
     end subroutine sqrm_solve_async
  end interface qrm_solve_async

  
  
  !> @brief Generic interface for the
  !! @link                                  ::sqrm_apply @endlink and
  !! @link                                  ::sqrm_apply1d @endlink routines
  !!
  interface qrm_apply
     procedure                              :: sqrm_apply2d, sqrm_apply1d
  end interface qrm_apply


  ! !> @brief Generic interface for the
  ! !! @link                                ::sqrm_apply @endlink and
  ! !! @link                                ::sqrm_apply1d @endlink routines
  ! !!
  interface sqrm_apply
     subroutine sqrm_apply2d(qrm_spfct, transp, b, info)
       use sqrm_spfct_mod
       type(sqrm_spfct_type), intent(in)    :: qrm_spfct
       real(r32), intent(inout)             :: b(:,:)
       character(len=*), intent(in)         :: transp
       integer, optional                    :: info
     end subroutine sqrm_apply2d
     subroutine sqrm_apply1d(qrm_spfct, transp, b, info)
       use sqrm_spfct_mod
       type(sqrm_spfct_type), intent(in)    :: qrm_spfct
       real(r32), intent(inout)             :: b(:)
       character(len=*), intent(in)         :: transp
       integer, optional                    :: info
     end subroutine sqrm_apply1d
  end interface sqrm_apply


  !> @brief Generic interface for the
  !! @link                                  ::sqrm_solve @endlink and
  !! @link                                  ::sqrm_solve1d @endlink routines
  !!
  interface qrm_solve
     procedure                              :: sqrm_solve1d, sqrm_solve2d
  end interface qrm_solve

  !> @brief Generic interface for the
  !! @link                                  ::sqrm_solve @endlink and
  !! @link                                  ::sqrm_solve1d @endlink routines
  !!
  interface sqrm_solve
     subroutine sqrm_solve2d(qrm_spfct, transp, b, x, info)
       use sqrm_spfct_mod
       type(sqrm_spfct_type), target        :: qrm_spfct
       real(r32), intent(inout)             :: b(:,:)
       real(r32), intent(out)               :: x(:,:)
       character(len=*)                     :: transp
       integer, optional                    :: info
     end subroutine sqrm_solve2d
     subroutine sqrm_solve1d(qrm_spfct, transp, b, x, info)
       use sqrm_spfct_mod
       type(sqrm_spfct_type), target        :: qrm_spfct
       real(r32), intent(inout)             :: b(:)
       real(r32), intent(out)               :: x(:)
       character(len=*)                     :: transp
       integer, optional                    :: info
     end subroutine sqrm_solve1d
  end interface sqrm_solve

  interface qrm_apply_node_task
     subroutine sqrm_apply_node_task(qrm_dscr, transp, qrm_spfct, inode, qrm_sdata_b)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       use qrm_dscr_mod
       type(qrm_dscr_type)                  :: qrm_dscr
       type(sqrm_spfct_type), target        :: qrm_spfct
       type(sqrm_sdata_type)                :: qrm_sdata_b
       character(len=*), intent(in)         :: transp
       integer                              :: inode
     end subroutine sqrm_apply_node_task
  end interface qrm_apply_node_task
  
  interface qrm_solve_node_task
     subroutine sqrm_solve_node_task(qrm_dscr, transp, qrm_spfct, inode, qrm_sdata_b, qrm_sdata_x)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       use qrm_dscr_mod
       type(qrm_dscr_type)                  :: qrm_dscr
       type(sqrm_spfct_type), target        :: qrm_spfct
       type(sqrm_sdata_type)                :: qrm_sdata_b, qrm_sdata_x
       character(len=*), intent(in)         :: transp
       integer                              :: inode
     end subroutine sqrm_solve_node_task
  end interface qrm_solve_node_task
  
  
  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Kernels
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  interface qrm_front_qt
     subroutine sqrm_front_qt(front, qrm_sdata_b, work)
       use sqrm_fdata_mod
       use sqrm_sdata_mod
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b
       type(sqrm_ws_type)                   :: work
     end subroutine sqrm_front_qt
  end interface qrm_front_qt

  interface qrm_assemble_qt
     subroutine sqrm_assemble_qt(qrm_spfct, front, qrm_sdata_b, info)
       use sqrm_spfct_mod
       use sqrm_fdata_mod
       use sqrm_sdata_mod
       type(sqrm_spfct_type)                :: qrm_spfct
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b
       integer, optional                    :: info
     end subroutine sqrm_assemble_qt
  end interface qrm_assemble_qt

  interface qrm_front_r
     subroutine sqrm_front_r(front, qrm_sdata_b, qrm_sdata_x)
       use sqrm_fdata_mod
       use sqrm_sdata_mod
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b, qrm_sdata_x
     end subroutine sqrm_front_r
  end interface qrm_front_r

  interface qrm_assemble_r
     subroutine sqrm_assemble_r(qrm_spfct, front, qrm_sdata_b, qrm_sdata_x, info)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       type(sqrm_spfct_type), target        :: qrm_spfct
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b, qrm_sdata_x
       integer, optional                    :: info
     end subroutine sqrm_assemble_r
  end interface qrm_assemble_r

  interface qrm_front_q
     subroutine sqrm_front_q(front, qrm_sdata_b, work)
       use sqrm_fdata_mod
       use sqrm_sdata_mod
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b
       type(sqrm_ws_type)                   :: work
     end subroutine sqrm_front_q
  end interface qrm_front_q

  interface qrm_assemble_q
     subroutine sqrm_assemble_q(qrm_spfct, front, qrm_sdata_b, info)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       type(sqrm_spfct_type), target        :: qrm_spfct
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b
       integer, optional                    :: info
     end subroutine sqrm_assemble_q
  end interface qrm_assemble_q

  interface qrm_front_rt
     subroutine sqrm_front_rt(front, qrm_sdata_b, qrm_sdata_x)
       use sqrm_fdata_mod
       use sqrm_sdata_mod
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b, qrm_sdata_x
     end subroutine sqrm_front_rt
  end interface qrm_front_rt

  interface qrm_assemble_rt
     subroutine sqrm_assemble_rt(qrm_spfct, front, qrm_sdata_b, qrm_sdata_x, info)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       type(sqrm_spfct_type), target        :: qrm_spfct
       type(sqrm_front_type)                :: front
       type(sqrm_sdata_type)                :: qrm_sdata_b, qrm_sdata_x
       integer, optional                    :: info
     end subroutine sqrm_assemble_rt
  end interface qrm_assemble_rt

  interface qrm_apply_subtree
     subroutine sqrm_apply_subtree(transp, qrm_spfct, iroot, qrm_sdata_b, work, info)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       type(sqrm_spfct_type), intent(in)    :: qrm_spfct
       type(sqrm_sdata_type)                :: qrm_sdata_b
       type(sqrm_ws_type), intent(inout)    :: work
       character(len=*), intent(in)         :: transp
       integer                              :: iroot
       integer, optional                    :: info
     end subroutine sqrm_apply_subtree
  end interface qrm_apply_subtree

  interface qrm_solve_subtree
     subroutine sqrm_solve_subtree(transp, qrm_spfct, iroot, qrm_sdata_b, qrm_sdata_x, info)
       use sqrm_spfct_mod
       use sqrm_sdata_mod
       type(sqrm_spfct_type), intent(in)    :: qrm_spfct
       type(sqrm_sdata_type)                :: qrm_sdata_b, qrm_sdata_x
       character(len=*), intent(in)         :: transp
       integer                              :: iroot
       integer, optional                    :: info
     end subroutine sqrm_solve_subtree
  end interface qrm_solve_subtree
  
end module sqrm_solve_mod
