!! ##############################################################################################
!!
!! Copyright 2012-2020 CNRS, INPT
!! Copyright 2013-2015 UPS
!!  
!! This file is part of qr_mumps.
!!  
!! qr_mumps is free software: you can redistribute it and/or modify
!! it under the terms of the GNU Lesser General Public License as 
!! published by the Free Software Foundation, either version 3 of 
!! the License, or (at your option) any later version.
!!  
!! qr_mumps is distributed in the hope that it will be useful,
!! but WITHOUT ANY WARRANTY; without even the implied warranty of
!! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!! GNU Lesser General Public License for more details.
!!  
!! You can find a copy of the GNU Lesser General Public License
!! in the qr_mumps/doc directory.
!!
!! ##############################################################################################


!! ##############################################################################################
!!
!! This file contains a module with all the generic interfaces for the typed methods routines.
!!
!! ##############################################################################################


!> This module contains generic methods
module zqrm_methods_mod

  interface qrm_spbackslash
     procedure               ::  zqrm_spbackslash2d, zqrm_spbackslash1d
  end interface qrm_spbackslash

  interface zqrm_spbackslash
     subroutine zqrm_spbackslash2d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_spbackslash2d
     subroutine zqrm_spbackslash1d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_spbackslash1d
  end interface zqrm_spbackslash

  
  interface qrm_spposv
     procedure               ::  zqrm_spposv2d, zqrm_spposv1d
  end interface qrm_spposv

  interface zqrm_spposv
     subroutine zqrm_spposv2d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_spposv2d
     subroutine zqrm_spposv1d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_spposv1d
  end interface zqrm_spposv

  interface qrm_least_squares
     procedure               ::  zqrm_least_squares2d, zqrm_least_squares1d
  end interface qrm_least_squares

  interface zqrm_least_squares
     subroutine zqrm_least_squares2d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_least_squares2d
     subroutine zqrm_least_squares1d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_least_squares1d
  end interface zqrm_least_squares
  


  interface qrm_min_norm
     procedure               :: zqrm_min_norm2d, zqrm_min_norm1d
  end interface qrm_min_norm

  interface zqrm_min_norm
     subroutine zqrm_min_norm2d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:,:), x(:,:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_min_norm2d
     subroutine zqrm_min_norm1d(qrm_mat, b, x, cperm, info)
       use zqrm_spmat_mod
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:), x(:)
       integer, optional     :: cperm(:)
       integer, optional     :: info
     end subroutine zqrm_min_norm1d
  end interface zqrm_min_norm
  

  interface qrm_residual_norm
     procedure               :: zqrm_residual_norm2d, zqrm_residual_norm1d
  end interface qrm_residual_norm

  interface zqrm_residual_norm
     subroutine zqrm_residual_norm2d(qrm_mat, b, x, nrm, info)
       use zqrm_spmat_mod
       real(r64)             :: nrm(:)
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine zqrm_residual_norm2d
     subroutine zqrm_residual_norm1d(qrm_mat, b, x, nrm, info)
       use zqrm_spmat_mod
       real(r64)             :: nrm
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine zqrm_residual_norm1d
  end interface zqrm_residual_norm



  interface qrm_residual_orth
     procedure               :: zqrm_residual_orth2d, zqrm_residual_orth1d
     procedure               :: zqrm_residual_and_orth2d, zqrm_residual_and_orth1d
  end interface qrm_residual_orth

  interface zqrm_residual_orth
     subroutine zqrm_residual_orth2d(qrm_mat, r, nrm, info)
       use zqrm_spmat_mod
       real(r64)             :: nrm(:)
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: r(:,:)
       integer, optional     :: info
     end subroutine zqrm_residual_orth2d
     subroutine zqrm_residual_orth1d(qrm_mat, r, nrm, info)
       use zqrm_spmat_mod
       real(r64)             :: nrm
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: r(:)
       integer, optional     :: info
     end subroutine zqrm_residual_orth1d
     subroutine zqrm_residual_and_orth2d(qrm_mat, b, x, nrm, info)
       use zqrm_spmat_mod
       real(r64)             :: nrm(:)
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:,:), x(:,:)
       integer, optional     :: info
     end subroutine zqrm_residual_and_orth2d
     subroutine zqrm_residual_and_orth1d(qrm_mat, b, x, nrm, info)
       use zqrm_spmat_mod
       real(r64)             :: nrm
       type(zqrm_spmat_type) :: qrm_mat
       complex(r64)             :: b(:), x(:)
       integer, optional     :: info
     end subroutine zqrm_residual_and_orth1d
  end interface zqrm_residual_orth


  interface qrm_get_r
     subroutine zqrm_get_r(qrm_spfct, r, info)
       use zqrm_spmat_mod
       use zqrm_spfct_mod
       type(zqrm_spfct_type)          :: qrm_spfct
       type(zqrm_spmat_type)          :: r
       integer, optional              :: info
     end subroutine zqrm_get_r
  end interface qrm_get_r
  
end module zqrm_methods_mod
